# standard imports
from enum import Enum
from random import randint


DATE_FORMAT = "%d-%m-%Y"
MOBILE_PATTERN = r'[6-9]+[0-9]{9}'

class Endpoints(str, Enum):
    APPOINTMENT_PINCODE = "appointment/sessions/public/calendarByPin"
    APPOINTMENT_DISTRICT = "appointment/sessions/public/calendarByDistrict"
    BASE = "https://cdn-api.co-vin.in/api/v2"
    CONFIRM_OTP = "auth/public/confirmOTP"
    DISTRICTS = "admin/location/districts"
    DOWNLOAD_CERT = "registration/certificate/public/download"
    GET_OTP = "auth/public/generateOTP"
    STATES = "admin/location/states"

    def __str__(self):
        return str.__str__(self)


class UserAgents(str, Enum):
    FF_88 = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:88.0) Gecko/20100101 Firefox/88.0"
    FF_87 = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:88.0) Gecko/20100101 Firefox/87.0"
    CHROME_92 = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64) Chrome/92.0 Safari/537.36"
    CHROME_91 = "Mozilla/5.0 (Windows NT 6.2) Chrome/91.0 Safari/537.36"

    @staticmethod
    def random():
        ua_list = [ua.value for ua in UserAgents]
        return ua_list[randint(0, len(ua_list)-1)]


class Languages(str, Enum):
    TAMIL = 'ta_IN'
    ENGLISH_US = 'en_US'
    ENGLISH_UK = 'en_UK'
    HINDI = 'hi_IN'

    def __str__(self):
        return str.__str__(self)



# standard libs
import hashlib
import re

# local imports
from .constants import MOBILE_PATTERN


def is_mobile_number_valid(mobile_no):
    return bool(re.match(MOBILE_PATTERN, mobile_no))


def hash_otp(otp: int):
    return hashlib.sha256(str(otp).encode()).hexdigest()



# Standard imports
import datetime
from abc import ABC, abstractmethod

# Third party imports
from httpx import Client, AsyncClient

# local imports
from .constants import DATE_FORMAT, Endpoints, UserAgents, Languages
from .exceptions import BadRequest, ServerError
from .utils import is_mobile_number_valid, hash_otp


class BaseAPI(ABC):
    """Base Class for API client"""
    @property
    def base_url(self):
        """returns base url of the API"""
        return Endpoints.BASE

    @abstractmethod
    def get_states(self, language: str):
        """get list of states"""
        raise NotImplementedError()

    @abstractmethod
    def get_districts(self, state_id: int, language: str):
        """get list of districts for a state"""
        raise NotImplementedError()

    @abstractmethod
    def get_available_sessions_by_pincode(self, pincode: int, date: datetime.date):
        """get planned vaccination sessions on a specific date in a given
        pincode
        """
        raise NotImplementedError()

    @abstractmethod
    def get_otp(self, mobile: str):
        """sends OTP to supplied mobile number and returns txn ID"""
        raise NotImplementedError()

    @abstractmethod
    def confirm_otp(self, txn_id: str, otp: int):
        """confirm otp"""
        raise NotImplementedError()

    @abstractmethod
    def get_certificate(self, token: str, beneficiary_id: str):
        """download certificate as pdf for beneficiary"""
        raise NotImplementedError()

    @classmethod
    def process_response(cls, status_code, msg):
        """check response status code and raise exception if required"""
        if 400 <= status_code < 500:
            raise BadRequest(f"{status_code} - {msg}")
        if 500 <= status_code <= 599:
            raise ServerError(f"{status_code} - {msg}")


class Cowin(BaseAPI):
    """Cowin API client

    This client makes blocking synchronous http request to the server.
    """

    def __init__(self):
        self.__client = Client(base_url=self.base_url)

    def get_states(self, language=Languages.ENGLISH_US):
        headers = {
            'User-Agent': UserAgents.random(),
            'Accept-Language': language
        }
        response = self.__client.get(Endpoints.STATES, headers=headers)
        self.process_response(response.status_code, response.text)
        return response.json()

    def get_districts(self, state_id: int, language="en_US"):
        headers = {
            'User-Agent': UserAgents.random(),
            'Accept-Language': language
        }
        endpoint = f"{Endpoints.DISTRICTS}/{state_id}"
        response = self.__client.get(endpoint, headers=headers)
        self.process_response(response.status_code, response.text)
        return response.json()

    def get_available_sessions_by_pincode(self, pincode: int, date=None):
        if not date:
            date = datetime.date.today().strftime(DATE_FORMAT)

        headers = {'User-Agent': UserAgents.random()}
        query = {'pincode': pincode, 'date': date}
        response = self.__client.get(
            Endpoints.APPOINTMENT_PINCODE,
            headers=headers,
            params=query
        )
        self.process_response(response.status_code, response.text)
        return response.json()

    def get_otp(self, mobile: str):
        if not is_mobile_number_valid(mobile):
            raise BadRequest("Invalid mobile number")
        headers = {'User-Agent': UserAgents.random()}
        payload = {'mobile': mobile}
        response = self.__client.post(
            Endpoints.GET_OTP,
            headers=headers,
            json=payload
        )
        self.process_response(response.status_code, response.text)
        return response.json()

    def confirm_otp(self, txn_id: str, otp: str):
        headers = {'User-Agent': UserAgents.random()}
        payload = {
            'otp': hash_otp(otp),
            'txnId': txn_id
        }
        response = self.__client.post(
            Endpoints.CONFIRM_OTP,
            headers=headers,
            json=payload
        )
        self.process_response(response.status_code, response.text)
        return response.json()

    def get_certificate(self, token: str, beneficiary_id: str) -> bytes:
        headers = {
            'User-Agent': UserAgents.random(),
            'Authorization': f'Bearer {token}'
        }
        params = {'beneficiary_reference_id': beneficiary_id}
        response = self.__client.get(
            Endpoints.DOWNLOAD_CERT,
            headers=headers,
            params=params
        )
        self.process_response(response.status_code, response.text)
        if response.headers['content-type'] != 'application/pdf':
            raise ServerError("Content is not PDF")
        return response.content


class AsyncCowin(BaseAPI):
    """Cowin API Client

    This client makes non-blocking asynchronous http request to server.
    """

    async def get_states(self, language=Languages.ENGLISH_US):
        headers = {
            'User-Agent': UserAgents.random(),
            'Accept-Language': language
        }
        async with AsyncClient(base_url=self.base_url) as client:
            response = await client.get(Endpoints.STATES, headers=headers)
        self.process_response(response.status_code, response.text)
        return response.json()

    async def get_districts(self, state_id: int, language="en_US"):
        headers = {
            'User-Agent': UserAgents.random(),
            'Accept-Language': language
        }
        endpoint = f"{Endpoints.DISTRICTS}/{state_id}"
        async with AsyncClient(base_url=self.base_url) as client:
            response = await client.get(endpoint, headers=headers)
        self.process_response(response.status_code, response.text)
        return response.json()

    async def get_available_sessions_by_pincode(self, pincode: int, date=None):
        if not date:
            date = datetime.date.today().strftime(DATE_FORMAT)

        headers = {'User-Agent': UserAgents.random()}
        query = {'pincode': pincode, 'date': date}
        async with AsyncClient(base_url=self.base_url) as client:
            response = await client.get(
                Endpoints.APPOINTMENT_PINCODE,
                headers=headers,
                params=query
            )
        self.process_response(response.status_code, response.text)
        return response.json()

    async def get_otp(self, mobile: str):
        if not is_mobile_number_valid(mobile):
            raise BadRequest("Invalid mobile number")
        headers = {'User-Agent': UserAgents.random()}
        payload = {'mobile': mobile}
        async with AsyncClient(base_url=self.base_url) as client:
            response = await client.post(
                Endpoints.GET_OTP,
                headers=headers,
                json=payload
            )
        self.process_response(response.status_code, response.text)
        return response.json()

    async def confirm_otp(self, txn_id: str, otp: int):
        headers = {'User-Agent': UserAgents.random()}
        payload = {
            'otp': hash_otp(otp),
            'txnId': txn_id
        }
        async with AsyncClient(base_url=self.base_url) as client:
            response = await client.post(
                Endpoints.CONFIRM_OTP,
                headers=headers,
                json=payload
            )
        self.process_response(response.status_code, response.text)
        return response.json()

    async def get_certificate(self, token: str, beneficiary_id: str) -> bytes:
        headers = {
            'User-Agent': UserAgents.random(),
            'Authorization': f'Bearer {token}'
        }
        params = {'beneficiary_reference_id': beneficiary_id}
        async with AsyncClient(base_url=self.base_url) as client:
            response = await client.get(
                Endpoints.DOWNLOAD_CERT,
                headers=headers,
                params=params
            )
        self.process_response(response.status_code, response.text)
        if response.headers['content-type'] != 'application/pdf':
            raise ServerError('Content is not PDF')
        return response.content



from setuptools import setup, find_packages

def get_long_description():
    """
    Return the README.
    """
    with open('README.md', encoding="utf8") as fd:
        return fd.read()

setup(
    name='cowin-async',
    version=1.1,
    description='Interact with CoWin APIs',
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    license="LGPL v3.0",
    author="Prasanna Venkadesh",
    author_email="prasmailme@gmail.com",
    python_requires='>=3.8',
    install_requires=['httpx==0.18.*'],
    extras_require={
        'develop': [
            'pytest==6.2.4',
            'pytest-asyncio==0.15.1',
            'pytest-cov==2.11.1',
            'pytest-httpx==0.12.0'
        ]
    },
    packages=find_packages(exclude=['tests']),
    project_urls={
        "Source": "https://gitlab.com/prashere/cowin-async"
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP",
        "Framework :: AsyncIO",
        "Framework :: Trio",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3 :: Only"
    ]
)
